@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -339,15 +339,6 @@ void FileAccess::Login(const char *user1,const char *pass1)
    ResetLocationData();
 }
 
-void FileAccess::AnonymousLogin()
-{
-   Close();
-   user.set(0);
-   pass.set(0);
-   pass_open=false;
-   ResetLocationData();
-}
-
 void FileAccess::ResetLocationData()
 {
    cwd.Set(default_cwd,false,0);
