@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -50,10 +50,16 @@ void  FileInfo::Merge(const FileInfo& f)
 {
    if(strcmp(basename_ptr(name),basename_ptr(f.name)))
       return;
-// int sim=defined&f.defined;
-   int dif=(~defined)&f.defined;
-   if(dif&MODE)
+   MergeInfo(f,~defined);
+}
+void  FileInfo::MergeInfo(const FileInfo& f,unsigned dif)
+{
+   dif&=f.defined;
+   if(dif&MODE) {
       SetMode(f.mode);
+      if(mode!=SYMLINK && mode!=REDIRECT)
+	 symlink.unset();
+   }
    if(dif&DATE || (defined&DATE && f.defined&DATE && f.date.ts_prec<date.ts_prec))
       SetDate(f.date,f.date.ts_prec);
    if(dif&SIZE)
@@ -298,6 +304,29 @@ void FileSet::SubtractSameType(const FileSet *set)
 	 Sub(i--);
    }
 }
+void FileSet::SubtractDirs(const FileSet *set)
+{
+   for(int i=0; i<fnum; i++)
+   {
+      if(!files[i]->TypeIs(FileInfo::DIRECTORY))
+	 continue;
+      FileInfo *f=set->FindByName(files[i]->name);
+      if(f && f->TypeIs(f->DIRECTORY))
+	 Sub(i--);
+   }
+}
+void FileSet::SubtractNotOlderDirs(const FileSet *set)
+{
+   for(int i=0; i<fnum; i++)
+   {
+      if(!files[i]->TypeIs(FileInfo::DIRECTORY)
+      || !files[i]->Has(FileInfo::DATE))
+	 continue;
+      FileInfo *f=set->FindByName(files[i]->name);
+      if(f && f->TypeIs(f->DIRECTORY) && f->NotOlderThan(files[i]->date))
+	 Sub(i--);
+   }
+}
 
 void FileSet::SubtractTimeCmp(bool (FileInfo::*cmp)(time_t) const,time_t t)
 {
@@ -373,14 +402,17 @@ void FileSet::ExcludeCompound()
    }
 }
 
-void FileSet::ExcludeUnaccessible()
+void FileSet::ExcludeUnaccessible(const char *user)
 {
    for(int i=0; i<fnum; i++)
    {
-      if(!(files[i]->defined&FileInfo::MODE) || !(files[i]->defined&FileInfo::TYPE))
+      if(!files[i]->Has(FileInfo::MODE) || !files[i]->Has(FileInfo::TYPE))
 	 continue;
-      if((files[i]->filetype==FileInfo::NORMAL    && !(files[i]->mode&0444))
-      || (files[i]->filetype==FileInfo::DIRECTORY && !(files[i]->mode&0444&(files[i]->mode<<2))))
+      unsigned mask=0444;
+      if(user && files[i]->Has(FileInfo::USER))
+	 mask=(!strcmp(files[i]->user,user)?0400:0044);
+      if((files[i]->TypeIs(FileInfo::NORMAL)    && !(files[i]->mode&mask))
+      || (files[i]->TypeIs(FileInfo::DIRECTORY) && !(files[i]->mode&mask&(files[i]->mode<<2))))
       {
 	 Sub(i);
 	 i--;
@@ -458,7 +490,7 @@ void FileSet::Count(int *d,int *f,int *s,int *o) const
 	 if(f) (*f)++; break;
       case(FileInfo::SYMLINK):
 	 if(s) (*s)++; break;
-      case(FileInfo::UNKNOWN):
+      default:
 	 if(o) (*o)++;
       }
    }
@@ -694,12 +726,6 @@ FileInfo::FileInfo(const FileInfo &fi)
    nlinks=fi.nlinks;
    longname.set(fi.longname);
 }
-FileInfo::FileInfo(const char *n)
-{
-   Init();
-   SetName(n);
-}
-
 FileInfo::~FileInfo()
 {
 }
@@ -771,9 +797,11 @@ drwxr-xr-x   4 lav      root         1024 Feb 22 15:32 lib
 lrwxrwxrwx   1 lav      root           33 Feb 14 17:45 ltconfig -> /usr/share/libtool/ltconfig
 NOTE: group may be missing.
 */
-FileInfo *FileInfo::parse_ls_line(const char *line_c,const char *tz)
+FileInfo *FileInfo::parse_ls_line(const char *line_c,int line_len,const char *tz)
 {
-   char *line=alloca_strdup(line_c);
+   char *line=string_alloca(line_len+1);
+   memcpy(line,line_c,line_len);
+   line[line_len]=0;
    char *next=0;
    FileInfo *fi=0; /* don't instantiate until we at least have something */
 #define FIRST_TOKEN strtok_r(line," \t",&next)
@@ -802,7 +830,7 @@ FileInfo *FileInfo::parse_ls_line(const char *line_c,const char *tz)
    case('p'): // pipe
    case('s'): // sock
    case('D'): // Door
-      return 0;  // ignore
+      // ignore them
    default:
       ERR;
    }
@@ -976,6 +1004,7 @@ void FileInfo::MakeLongName()
    case UNKNOWN:   break;
    case DIRECTORY: filetype_c='d'; break;
    case SYMLINK:   filetype_c='l'; break;
+   case REDIRECT:  filetype_c='L'; break;
    }
    int mode1=(defined&MODE?mode:
       (filetype_c=='d'?0755:(filetype_c=='l'?0777:0644)));
