@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2015 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -91,6 +91,8 @@ void FileSetOutput::print(FileSet &fs, const JobRef<OutputJob>& o) const
 	 else mode[0] = '-';
 
 	 c.add(mode, "");
+      } else if(have & FileInfo::MODE) {
+	 c.add("           ", "");
       }
 
       if((have & FileInfo::NLINKS) && (mode & NLINKS)) {
