@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2015 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -222,6 +222,7 @@ void Http::MoveConnectionHere(Http *o)
 
 void Http::DisconnectLL()
 {
+   Enter(this);
    rate_limit=0;
    if(conn)
    {
@@ -247,6 +248,7 @@ void Http::DisconnectLL()
    last_url.unset();
    ResetRequestData();
    state=DISCONNECTED;
+   Leave(this);
 }
 
 void Http::ResetRequestData()
@@ -308,7 +310,7 @@ void Http::Close()
    no_cache_this=false;
    auth_sent[0]=auth_sent[1]=0;
    auth_scheme[0]=auth_scheme[1]=HttpAuth::NONE;
-   no_ranges=false;
+   no_ranges=!QueryBool("use-range",hostname);
    use_propfind_now=QueryBool("use-propfind",hostname);
    special=HTTP_NONE;
    special_data.set(0);
@@ -469,7 +471,7 @@ void Http::SendAuth()
       SendBasicAuth("Authorization",user,pass);
       return;
    }
-   SendAuth(HttpAuth::WWW,user,last_uri);
+   SendAuth(HttpAuth::WWW,user?user:auth_user,last_uri);
 }
 void Http::SendCacheControl()
 {
@@ -752,6 +754,7 @@ void Http::SendRequest(const char *connection,const char *f)
 	    weekday_names[t->tm_wday],t->tm_mday,month_names[t->tm_mon],
 	    t->tm_year+1900,t->tm_hour,t->tm_min,t->tm_sec);
 	 Send("Last-Modified: %s\r\n",d);
+	 Send("X-OC-MTime: %ld\r\n",(long)entity_date);	 // for OwnCloud
       }
       break;
 
@@ -841,7 +844,7 @@ void Http::SendRequest(const char *connection,const char *f)
    chunk_pos=0;
    request_pos=0;
    inflate=0;
-   no_ranges=false;
+   no_ranges=!QueryBool("use-range",hostname);
 
    conn->send_buf->SetPos(0);
 }
@@ -877,6 +880,10 @@ int Http::SendArrayInfoRequest()
 	 name=&xstring::get_tmp(*name);
 	 name->append('/');
       }
+      if(fi->uri)
+	 file_url.set(dir_file(GetConnectURL(),fi->uri));
+      else
+	 file_url.unset();
       SendRequest(array_send==fileset_for_info->count()-1 ? 0 : "keep-alive", *name);
       req_count++;
    }
@@ -1082,22 +1089,10 @@ void Http::HandleHeaderLine(const char *name,const char *value)
    case_hh("WWW-Authenticate",'W') {
       if(status_code!=H_Unauthorized)
 	 return;
-      const char *user=this->user;
-      const char *pass=this->pass;
-      if(!user || !pass) {
-	 // try auth info from http:authorization setting
-	 const char *auth_c=Query("authorization",hostname);
-         if(auth_c && *auth_c) {
-	    char *auth=alloca_strdup(auth_c);
-	    char *colon=strchr(auth,':');
-	    if(colon) {
-	       *colon=0;
-	       user=auth;
-	       pass=colon+1;
-	    }
-	 }
-      }
-      NewAuth(value,HttpAuth::WWW,user,pass);
+      if(user && pass)
+	 NewAuth(value,HttpAuth::WWW,user,pass);
+      else
+	 NewAuth(value,HttpAuth::WWW,auth_user,auth_pass);
       return;
    }
    case_hh("Proxy-Authenticate",'P') {
@@ -1578,6 +1573,14 @@ int Http::Do()
 		     }
 		     goto pre_RECEIVING_BODY;
 		  }
+		  FileInfo *fi=fileset_for_info->curr();
+		  if(H_REDIRECTED(status_code)) {
+		     HandleRedirection();
+		     if(location)
+			fi->SetRedirect(location);
+		  } else if(H_2XX(status_code) && !fi->Has(fi->TYPE)) {
+		     fi->SetType(last_uri.last_char()=='/'?fi->DIRECTORY:fi->NORMAL);
+		  }
 		  ProceedArrayInfo();
 		  return MOVED;
 	       }
@@ -1736,49 +1739,7 @@ int Http::Do()
 
 	 if(H_REDIRECTED(status_code))
 	 {
-	    bool is_url=(location && url::is_url(location));
-	    if(location && !is_url
-	    && mode==QUOTE_CMD && !strncasecmp(file,"POST ",5)
-	    && tunnel_state!=TUNNEL_WAITING)
-	    {
-	       const char *the_file=file;
-
-	       const char *scan=file+5;
-	       while(*scan==' ')
-		  scan++;
-	       char *the_post_file=alloca_strdup(scan);
-	       char *space=strchr(the_post_file,' ');
-	       if(space)
-		  *space=0;
-	       the_file=the_post_file;
-
-	       char *new_location=alloca_strdup2(GetConnectURL(),
-				    strlen(the_file)+strlen(location));
-	       int p_ind=url::path_index(new_location);
-	       if(location[0]=='/')
-		  strcpy(new_location+p_ind,location);
-	       else
-	       {
-		  if(the_file[0]=='/')
-		     strcpy(new_location+p_ind,the_file);
-		  else
-		  {
-		     char *slash=strrchr(new_location,'/');
-		     strcpy(slash+1,the_file);
-		  }
-		  char *slash=strrchr(new_location,'/');
-		  strcpy(slash+1,location);
-	       }
-	       location.set(new_location);
-	    } else if(is_url && !hftp) {
-	       ParsedURL url(location);
-	       if(url.proto.eq(GetProto()) && !xstrcasecmp(url.host,hostname)
-	       && user && !url.user) {
-		  // use the same user name after redirect to the same site.
-		  url.user.set(user);
-		  location.set_allocated(url.Combine());
-	       }
-	    }
+	    HandleRedirection();
 	    err.setf("%s (%s -> %s)",status+status_consumed,file.get(),
 				    location?location.get():"nowhere");
 	    code=FILE_MOVED;
@@ -1927,6 +1888,53 @@ system_error:
    return MOVED;
 }
 
+void Http::HandleRedirection()
+{
+   bool is_url=(location && url::is_url(location));
+   if(location && !is_url
+   && mode==QUOTE_CMD && !strncasecmp(file,"POST ",5)
+   && tunnel_state!=TUNNEL_WAITING)
+   {
+      const char *the_file=file;
+
+      const char *scan=file+5;
+      while(*scan==' ')
+	 scan++;
+      char *the_post_file=alloca_strdup(scan);
+      char *space=strchr(the_post_file,' ');
+      if(space)
+	 *space=0;
+      the_file=the_post_file;
+
+      char *new_location=alloca_strdup2(GetConnectURL(),
+			   strlen(the_file)+strlen(location));
+      int p_ind=url::path_index(new_location);
+      if(location[0]=='/')
+	 strcpy(new_location+p_ind,location);
+      else
+      {
+	 if(the_file[0]=='/')
+	    strcpy(new_location+p_ind,the_file);
+	 else
+	 {
+	    char *slash=strrchr(new_location,'/');
+	    strcpy(slash+1,the_file);
+	 }
+	 char *slash=strrchr(new_location,'/');
+	 strcpy(slash+1,location);
+      }
+      location.set(new_location);
+   } else if(is_url && !hftp) {
+      ParsedURL url(location);
+      if(url.proto.eq(GetProto()) && !xstrcasecmp(url.host,hostname)
+      && user && !url.user) {
+	 // use the same user name after redirect to the same site.
+	 url.user.set(user);
+	 location.set_allocated(url.Combine());
+      }
+   }
+}
+
 FileAccess *Http::New() { return new Http(); }
 FileAccess *HFtp::New() { return new HFtp(); }
 
@@ -1964,6 +1972,7 @@ int Http::Read(Buffer *buf,int size)
    int res=DO_AGAIN;
    if(state==RECEIVING_BODY && real_pos>=0)
    {
+      Enter(this);
       res=_Read(buf,size);
       if(res>0)
       {
@@ -1972,6 +1981,7 @@ int Http::Read(Buffer *buf,int size)
 	    rate_limit->BytesGot(res);
 	 TrySuccess();
       }
+      Leave(this);
    }
    return res;
 }
@@ -2303,6 +2313,7 @@ void Http::Reconfig(const char *name)
 
    user_agent=ResMgr::Query("http:user-agent",c);
    use_propfind_now=(use_propfind_now && QueryBool("use-propfind",c));
+   no_ranges=(no_ranges || !QueryBool("use-range",hostname));
 
    allprop=	// PROPFIND request
       "<?xml version=\"1.0\" ?>"
@@ -2312,6 +2323,20 @@ void Http::Reconfig(const char *name)
    if(!QueryBool("use-allprop",c))
       allprop="";
    allprop_len=strlen(allprop);
+
+   if(!user || !pass) {
+      // get auth info from http:authorization setting
+      const char *auth_c=Query("authorization",hostname);
+      if(auth_c && *auth_c) {
+	 char *auth=alloca_strdup(auth_c);
+	 char *colon=strchr(auth,':');
+	 if(colon) {
+	    *colon=0;
+	    auth_user.set(auth);
+	    auth_pass.set(colon+1);
+	 }
+      }
+   }
 }
 
 bool Http::SameSiteAs(const FileAccess *fa) const
@@ -2339,6 +2364,7 @@ void Http::ResetLocationData()
    Reconfig();
    state=DISCONNECTED;
    use_propfind_now=QueryBool("use-propfind",hostname);
+   no_ranges=!QueryBool("use-range",hostname);
 }
 
 DirList *Http::MakeDirList(ArgV *args)
