@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2015 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -273,6 +273,7 @@ void  MirrorJob::HandleFile(FileInfo *file)
    switch(filetype)
    {
       case(FileInfo::NORMAL):
+      case(FileInfo::REDIRECT):
       {
 	 bool remove_target=false;
 	 bool cont_this=false;
@@ -1215,7 +1216,7 @@ int   MirrorJob::Do()
 	 break;
 
       // all jobs finished.
-      if(remove_source_dirs) {
+      if(remove_source_dirs && source_dir.last_char()!='/') {
 	 // remove source directory once.
 	 remove_source_dirs=false;
 	 if(script)
