@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2015 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -1002,8 +1002,10 @@ void SFtp::HandleExpect(Expect *e)
    case Expect::WRITE_STATUS:
       if(reply->TypeIs(SSH_FXP_STATUS))
       {
-	 if(((Reply_STATUS*)reply)->GetCode()==SSH_FX_OK)
+	 if(((Reply_STATUS*)reply)->GetCode()==SSH_FX_OK) {
+	    TrySuccess();
 	    break;
+	 }
       }
       SetError(NO_FILE,reply);
       break;
@@ -1026,6 +1028,7 @@ void SFtp::HandleExpect(Expect *e)
 
 void SFtp::RequestMoreData()
 {
+   Enter(this);
    if(mode==RETRIEVE) {
       int req_len=size_read;
       SendRequest(new Request_READ(handle,request_pos,req_len),Expect::DATA);
@@ -1033,6 +1036,7 @@ void SFtp::RequestMoreData()
    } else if(mode==LIST || mode==LONG_LIST) {
       SendRequest(new Request_READDIR(handle),Expect::DATA);
    }
+   Leave(this);
 }
 
 int SFtp::HandleReplies()
@@ -1209,7 +1213,9 @@ int SFtp::Write(const void *buf,int size)
       return(0);
 
    Resume();
+   Enter(this);
    Do();
+   Leave(this);
    if(Error())
       return(error_code);
 
@@ -1231,7 +1237,6 @@ int SFtp::Write(const void *buf,int size)
    if(size<=0)
       return 0;
    file_buf->Put(static_cast<const char*>(buf),size);
-   TrySuccess();
    rate_limit->BytesPut(size);
    pos+=size;
    real_pos+=size;
