@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2015 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -72,8 +72,9 @@ int SSH_Access::HandleSSHMessage()
       }
       if(s>=y_len && !strncasecmp(b+s-y_len,y,y_len))
       {
-	 pty_recv_buf->Put("yes\n");
-	 pty_send_buf->Put("yes\n");
+	 const char *answer=QueryBool("auto-confirm",hostname)?"yes\n":"no\n";
+	 pty_recv_buf->Put(answer);
+	 pty_send_buf->Put(answer);
 	 return m;
       }
       if(!received_greeting && recv_buf->Size()>0)
@@ -99,6 +100,15 @@ int SSH_Access::HandleSSHMessage()
       SetError(FATAL,_(f));
       return MOVED;
    }
+   if(eol>b && eol[-1]=='\r')
+      eol--;
+   f=N_("Name or service not known");
+   int f_len=strlen(f);
+   if(eol-b>=f_len && !strncasecmp(eol-f_len,f,f_len)) {
+      LogSSHMessage();
+      SetError(LOOKUP_ERROR,xstring::get_tmp(b,eol-b));
+      return MOVED;
+   }
    LogSSHMessage();
    return MOVED;
 }
@@ -118,21 +128,28 @@ void SSH_Access::LogSSHMessage()
 	 LogError(0,_("Peer closed connection"));
       }
       if(pty_recv_buf->Error())
-	 LogError(0,"pty read: %s",pty_recv_buf->ErrorText());
-      if(pty_recv_buf->Eof() || pty_recv_buf->Error())
-	 Disconnect();
+	 LogError(4,"pty read: %s",pty_recv_buf->ErrorText());
+      if(pty_recv_buf->Eof() || pty_recv_buf->Error()) {
+	 if(last_ssh_message && time_t(now)-last_ssh_message_time<4)
+	    LogError(0,"%s",last_ssh_message.get());
+	 Disconnect(last_ssh_message);
+      }
       return;
    }
    s=eol-b+1;
-   xstring &line=xstring::get_tmp(b,s-1);
+   int chomp_cr=(s>=2 && b[s-2]=='\r');
+   last_ssh_message.nset(b,s-1-chomp_cr);
+   last_ssh_message_time=now;
    pty_recv_buf->Skip(s);
-   LogRecv(4,line);
+   LogRecv(4,last_ssh_message);
+   if(last_ssh_message.begins_with("ssh: "))
+      last_ssh_message.set(last_ssh_message+5);
 
-   if(!received_greeting && line.eq(greeting))
+   if(!received_greeting && last_ssh_message.eq(greeting))
       received_greeting=true;
 }
 
-void SSH_Access::Disconnect()
+void SSH_Access::DisconnectLL()
 {
    if(send_buf)
       LogNote(9,_("Disconnecting"));
@@ -143,6 +160,8 @@ void SSH_Access::Disconnect()
    ssh=0;
    received_greeting=false;
    password_sent=0;
+   last_ssh_message.unset();
+   last_ssh_message_time=0;
 }
 
 void SSH_Access::MoveConnectionHere(SSH_Access *o)
@@ -154,4 +173,6 @@ void SSH_Access::MoveConnectionHere(SSH_Access *o)
    ssh=o->ssh.borrow();
    received_greeting=o->received_greeting;
    password_sent=o->password_sent;
+   last_ssh_message.move_here(o->last_ssh_message);
+   last_ssh_message_time=o->last_ssh_message_time; o->last_ssh_message_time=0;
 }
