@@ -35,9 +35,6 @@ CDECL_BEGIN
 CDECL_END
 #endif
 
-#define GET_BUFSIZE 0x10000
-#define PUT_LL_MIN  0x2000
-
 class Buffer
 {
 protected:
@@ -94,6 +91,9 @@ public:
    }
    void Prepend(const char *buf,int size);
    void Prepend(const char *buf) { Prepend(buf,strlen(buf)); }
+   int MoveDataHere(Buffer *o,int len);
+   template<class BUF> int MoveDataHere(const Ref<BUF>& o,int len) { return MoveDataHere(o.get_non_const(),len); }
+   template<class BUF> int MoveDataHere(const SMTaskRef<BUF>& o,int len) { return MoveDataHere(o.get_non_const(),len); }
 
    unsigned long long UnpackUINT64BE(int offset=0) const;
    unsigned UnpackUINT32BE(int offset=0) const;
@@ -169,6 +169,7 @@ protected:
 public:
    DirectedBuffer(dir_t m) : mode(m) {}
    void SetTranslator(DataTranslator *t);
+   const Ref<DataTranslator>& GetTranslator() const { return translator; }
    void SetTranslation(const char *be_encoding,bool translit=true)
 #ifdef HAVE_ICONV
       ;
@@ -183,6 +184,9 @@ public:
    void PutRaw(const char *buf) { Buffer::Put(buf); }
    void Put(const char *buf,int size);
    void Put(const char *buf) { Put(buf,strlen(buf)); }
+   void PutEOF(); // set eof, flush translator
+   int MoveDataHere(Buffer *o,int len);
+   template<class BUF> int MoveDataHere(const SMTaskRef<BUF>& o,int len) { return MoveDataHere(o.get_non_const(),len); }
    dir_t GetDirection() { return mode; }
 };
 
@@ -196,10 +200,18 @@ protected:
 
    Time event_time; // used to detect timeouts
    int max_buf;
+   int get_size;
+   int TuneGetSize(int res);
+
+   enum {
+      GET_BUFSIZE=0x10000,
+      PUT_LL_MIN=0x2000,
+   };
+
+   virtual ~IOBuffer();
 
 public:
    IOBuffer(dir_t m);
-   virtual ~IOBuffer();
    virtual const Time& EventTime()
       {
 	 if(IsSuspended())
@@ -240,6 +252,7 @@ class IOBufferStacked : public IOBuffer
 
 public:
    IOBufferStacked(IOBuffer *b) : IOBuffer(b->GetDirection()), down(b) {}
+   void PrepareToDie() { down=0; }
    const Time& EventTime() { return down->EventTime(); }
    int Do();
    bool Done();
@@ -286,7 +299,7 @@ public:
    IOBufferFileAccess(FileAccess *fa) : IOBuffer(GET), session(session_ref), session_ref(fa) {}
    ~IOBufferFileAccess() {
       // we don't want to delete the session
-      session_ref.borrow();
+      (void)session_ref.borrow();
    }
 
    const char *Status();
