@@ -95,6 +95,7 @@ class Ftp : public NetAccess
       bool received_150;
 
       char type;  // type of transfer: 'A'scii or 'I'mage
+      char t_mode; // transfer mode: 'S'tream, 'Z'ipped
 
       bool dos_path;
       bool vms_path;
@@ -113,8 +114,10 @@ class Ftp : public NetAccess
       bool clnt_supported;
       bool host_supported;
       bool mfmt_supported;
+      bool mff_supported;
       bool epsv_supported;
       bool tvfs_supported;
+      bool mode_z_supported;
 
       off_t last_rest;	// last successful REST position.
       off_t rest_pos;	// the number sent with REST command.
@@ -144,6 +147,7 @@ class Ftp : public NetAccess
 #endif
 
       xstring_c mlst_attr_supported;
+      xstring_c mode_z_opts_supported;
 
       Connection(const char *c);
       ~Connection();
@@ -170,6 +174,9 @@ class Ftp : public NetAccess
       void SendCmdF(const char *fmt,...) PRINTF_LIKE(2,3);
       int FlushSendQueueOneCmd(); // sends single command from send_cmd_buffer
 
+      void AddDataTranslator(DataTranslator *t);
+      void AddDataTranslation(const char *charset,bool translit);
+
       void SuspendInternal()
       {
 	 if(control_send) control_send->SuspendSlave();
@@ -183,6 +190,7 @@ class Ftp : public NetAccess
 	 if(data_iobuf)	data_iobuf->ResumeSlave();
       }
 
+      void CheckFEAT(char *reply,const char *line,bool trust);
    };
 
    Ref<Connection> conn;
@@ -197,6 +205,7 @@ class Ftp : public NetAccess
 	 READY,		// check response after connect
 	 REST,		// check response for REST
 	 TYPE,		// check response for TYPE
+	 MODE,
 	 CWD,		// check response for CWD
 	 CWD_CURR,	// check response for CWD into current directory
 	 CWD_STALE,	// check response for CWD when it's not critical
@@ -233,10 +242,11 @@ class Ftp : public NetAccess
       };
 
       expect_t check_case;
+      xstring_c cmd;
       xstring_c arg;
       Expect *next;
 
-      Expect(expect_t e,const char *a=0) : check_case(e), arg(a) {}
+      Expect(expect_t e,const char *a=0,const char *c=0) : check_case(e), cmd(c), arg(a) {}
       Expect(expect_t e,char c) : check_case(e)
 	 {
 	    char cc[2]={c,0};
@@ -279,7 +289,6 @@ class Ftp : public NetAccess
    void	 NoPassReqCheck(int);
    void	 proxy_LoginCheck(int);
    void	 proxy_NoPassReqCheck(int);
-   void	 CheckFEAT(char *reply);
    char *ExtractPWD();
    int   SendCWD(const char *path,const char *path_url,Expect::expect_t c);
    void	 CatchDATE(int);
@@ -327,6 +336,7 @@ private:
    void	SendSiteIdle();
    void	SendAcct();
    void	SendSiteGroup();
+   void	SendSiteCommands();
    void	SendUTimeRequest();
    void SendAuth(const char *auth);
    void TuneConnectionAfterFEAT();
@@ -423,8 +433,10 @@ private:
 
    int CanRead();
 
+   const char *path_to_send();
+
 protected:
-   ~Ftp();
+   void PrepareToDie();
 
 public:
    static void ClassInit();
