@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,12 +17,10 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: lftp_rl.c,v 1.20 2010/11/24 07:46:45 lav Exp $ */
-
 #include <config.h>
 #include <stdio.h>
-#include <readline/readline.h>
-#include <readline/history.h>
+#include <readline.h>
+#include <history.h>
 #include <stdlib.h>
 #include <string.h>
 #include "lftp_rl.h"
@@ -196,7 +194,7 @@ void lftp_rl_set_prompt(const char *p)
    rl_set_prompt(p);
 }
 
-extern char *get_lftp_home();
+extern char *get_lftp_data_dir();
 
 static char *lftp_history_file;
 void lftp_rl_read_history()
@@ -204,7 +202,7 @@ void lftp_rl_read_history()
    if(!lftp_history_file)
    {
       const char *add="/rl_history";
-      const char *home=get_lftp_home();
+      const char *home=get_lftp_data_dir();
       if(!home)
 	 return;
       lftp_history_file=(char*)malloc(strlen(home)+strlen(add)+1);
