@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,9 +23,9 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdarg.h>
-#include "SMTask.h"
+#include "Ref.h"
 
-class Log : public SMTask
+class Log
 {
    int output;
    bool need_close_output;
@@ -44,14 +44,14 @@ class Log : public SMTask
 	 output=-1;
 	 need_close_output=false;
       }
-   bool WillOutput(int l);
-
-public:
-   static SMTaskRef<Log> global;
 
    bool enabled;
    int level;
 
+public:
+   static Ref<Log> global;
+
+   bool WillOutput(int l);
    void DoWrite(const char *str);
    void Write(int l,const char *str);
    void Format(int l,const char *fmt,...) PRINTF_LIKE(3,4);
@@ -67,8 +67,7 @@ public:
 
    bool IsTTY() { return tty; }
 
-   void Init();
-   Log() { Init(); }
+   Log();
    ~Log();
 
    int Do();
