@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2015 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -773,6 +773,9 @@ bool temporary_network_error(int err)
 #endif
 #ifdef ECONNABORTED
    case(ECONNABORTED):
+#endif
+#ifdef EADDRNOTAVAIL
+   case(EADDRNOTAVAIL):
 #endif
       return true;
    }
