@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2015 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -284,6 +284,7 @@ static ResType lftp_vars[] = {
    {"hftp:use-head",		 "yes",   ResMgr::BoolValidate,0},
    {"hftp:use-mkcol",		 "no",	  ResMgr::BoolValidate,0},
    {"hftp:use-propfind",	 "no",	  ResMgr::BoolValidate,0},
+   {"hftp:use-range",		 "yes",   ResMgr::BoolValidate,0},
    {"hftp:use-allprop",		 "no",	  ResMgr::BoolValidate,0},
    {"hftp:use-type",		 "yes",   ResMgr::BoolValidate,0},
    {"http:accept",		 "*/*",   0,0},
@@ -297,6 +298,7 @@ static ResType lftp_vars[] = {
    {"http:proxy",		 "",	  HttpProxyValidate,0},
    {"http:use-mkcol",		 "yes",   ResMgr::BoolValidate,0},
    {"http:use-propfind",	 "no",    ResMgr::BoolValidate,0},
+   {"http:use-range",		 "yes",   ResMgr::BoolValidate,0},
    {"http:use-allprop",		 "no",	  ResMgr::BoolValidate,0},
    {"http:user-agent",		 PACKAGE "/" VERSION,0,0},
    {"http:cookie",		 "",	  0,0},
@@ -389,6 +391,7 @@ static ResType lftp_vars[] = {
    {"xfer:max-log-size",	 "1M",	  ResMgr::UNumberValidate,ResMgr::NoClosure},
    {"xfer:use-temp-file",	 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
    {"xfer:temp-file-name",	 ".in.*", 0,ResMgr::NoClosure},
+   {"xfer:timeout",		 "1d",	  ResMgr::TimeIntervalValidate,ResMgr::NoClosure},
    {"xfer:clobber",		 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
    {"xfer:make-backup",		 "yes",	  ResMgr::BoolValidate,ResMgr::NoClosure},
    {"xfer:keep-backup",		 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
