@@ -1,7 +1,7 @@
 /*
- * lftp and utils
+ * lftp - file transfer program
  *
- * Copyright (c) 1996-2008 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,8 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -29,6 +28,7 @@
 #include "bookmark.h"
 #include "misc.h"
 #include "log.h"
+#include "network.h"
 
 /*
    URL -> [PROTO://]CONNECT[[:]/PATH]
@@ -215,20 +215,20 @@ decode:
       if(!bm || !bm[0])
 	 return;
       const char *new_url=0;
+      xstring u(bm);
       if(orig_url)
       {
 	 const char *new_path=orig_url+url::path_index(orig_url);
 	 if(new_path[0]=='/')
 	    new_path++;
-	 char *u=alloca_strdup2(bm,strlen(new_path)+1);
 	 if(new_path[0]=='/' || new_path[0]=='~')
-	    u[url::path_index(u)]=0;
+	    u.truncate(url::path_index(u));
 	 assert(u[0]);
-	 if(u[strlen(u)-1]!='/' && new_path[0]!='/')
-	    strcat(u,"/");
-	 else if(u[strlen(u)-1]=='/' && new_path[0]=='/')
+	 if(u.last_char()!='/' && new_path[0]!='/')
+	    u.append('/');
+	 else if(u.last_char()=='/' && new_path[0]=='/')
 	    new_path++;
-	 strcat(u,new_path);
+	 u.append(new_path);
 	 new_url=u;
       }
       else
@@ -313,8 +313,15 @@ char *ParsedURL::Combine(const char *home,bool use_rfc1738)
       }
       u.append('@');
    }
-   if(host && !is_file)
-      u.append(url::encode(host,URL_HOST_UNSAFE));
+   if(host && !is_file) {
+      unsigned encode_flags=0;
+      if(xtld_name_ok(host))
+	 encode_flags|=URL_ALLOW_8BIT;
+      if(is_ipv6_address(host))
+	 u.append('[').append(host).append(']');
+      else
+	 u.append_url_encoded(host,URL_HOST_UNSAFE,encode_flags);
+   }
    if(port && !is_file)
    {
       u.append(':');
@@ -341,17 +348,17 @@ char *ParsedURL::Combine(const char *home,bool use_rfc1738)
    return u.borrow();
 }
 
-xstring& url::decode(const char *p)
+const xstring& url::decode(const char *p)
 {
    if(!p)
       return xstring::null;
    return xstring::get_tmp(p).url_decode();
 }
-xstring& url::encode(const char *s,int len,const char *unsafe)
+const xstring& url::encode(const char *s,int len,const char *unsafe,unsigned flags)
 {
    if(!s)
       return xstring::null;
-   return xstring::get_tmp("").append_url_encoded(s,len,unsafe);
+   return xstring::get_tmp("").append_url_encoded(s,len,unsafe,flags);
 }
 
 bool url::dir_needs_trailing_slash(const char *proto_c)
