@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2016 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,37 +23,9 @@
 #include "xstring.h"
 #include "trio.h"
 #include "c-ctype.h"
-
-int xstrcmp(const char *s1,const char *s2)
-{
-   if(s1==s2)
-      return 0;
-   if(s1==0 || s2==0)
-      return 1;
-   return strcmp(s1,s2);
-}
-int xstrncmp(const char *s1,const char *s2,size_t len)
-{
-   if(s1==s2 || len==0)
-      return 0;
-   if(s1==0 || s2==0)
-      return 1;
-   return strncmp(s1,s2,len);
-}
-int xstrcasecmp(const char *s1,const char *s2)
-{
-   if(s1==s2)
-      return 0;
-   if(s1==0 || s2==0)
-      return 1;
-   return strcasecmp(s1,s2);
-}
-size_t xstrlen(const char *s)
-{
-   if(s==0)
-      return 0;
-   return strlen(s);
-}
+CDECL_BEGIN
+#include "memcasecmp.h"
+CDECL_END
 
 void xstring::get_space(size_t s)
 {
@@ -212,6 +184,36 @@ bool xstring::eq(const char *o_buf,size_t o_len) const
 {
    return len==o_len && begins_with(o_buf,o_len);
 }
+bool xstring::eq_nc(const char *o_buf,size_t o_len) const
+{
+   if(len!=o_len)
+      return false;
+   if(buf==o_buf)
+      return true;
+   if(!buf || !o_buf)
+      return false;
+   return !memcasecmp(buf,o_buf,o_len);
+}
+int xstring::cmp(const char *o_buf,size_t o_len) const
+{
+   if(buf!=o_buf) {
+      if(!buf)
+	 return -1;
+      if(!o_buf)
+	 return 1;
+      size_t cmp_len=len;
+      if(cmp_len>o_len)
+	 cmp_len=o_len;
+      if(cmp_len>0) {
+	 int cmp_res=memcmp(buf,o_buf,cmp_len);
+	 if(cmp_res)
+	    return cmp_res;
+      }
+   }
+   if(len==o_len)
+      return 0;
+   return len-o_len;
+}
 
 static size_t vstrlen(va_list va0)
 {
@@ -340,14 +342,16 @@ xstring& xstring::vappendf(const char *format, va_list ap)
    {
       va_list tmp;
       VA_COPY(tmp,ap);
-      size_t res=vsnprintf(buf+len, size-len, format, tmp);
+      int res=vsnprintf(buf+len, size-len, format, tmp);
       va_end(tmp);
-      if(res>=0 && res<size-len)
+      if(res<0)
+	 return *this; // error
+      if((size_t)res<size-len)
       {
 	 set_length(len+res);
 	 return *this;
       }
-      get_space(res>size-len ? len+res+1 : len+(size-len)*2);
+      get_space((size_t)res>size-len ? len+res+1 : len+(size-len)*2);
    }
 }
 xstring& xstring::setf(const char *format, ...)
@@ -575,8 +579,10 @@ xstring& xstring::hex_decode()
 }
 
 /* Encode the unsafe characters in a given string, producing %XX encoded string. */
-#define need_quote(c) (c_iscntrl((c)) || !c_isascii((c)) || strchr(unsafe,(c)))
-xstring& xstring::append_url_encoded(const char *s,int len,const char *unsafe)
+#define need_quote(c) (c_iscntrl((c)) \
+   || (!(flags&URL_ALLOW_8BIT) && !c_isascii((c))) \
+   || strchr(unsafe,(c)))
+xstring& xstring::append_url_encoded(const char *s,int len,const char *unsafe,unsigned flags)
 {
    if(!s)
       return *this;
@@ -592,4 +598,30 @@ xstring& xstring::append_url_encoded(const char *s,int len,const char *unsafe)
    return *this;
 }
 
-xstring xstring::null;
+xstring& xstring::c_lc() {
+   char *p=buf;
+   int r=len;
+   while(r>0) {
+      *p=c_tolower(*p);
+      p++,r--;
+   }
+   return *this;
+}
+xstring& xstring::c_ucfirst() {
+   char *p=buf;
+   int r=len;
+   bool first=true;
+   while(r>0) {
+      if(c_isalpha(*p)) {
+	 *p=(first?c_toupper(*p):c_tolower(*p));
+	 first=false;
+      } else {
+	 first=true;
+      }
+      p++,r--;
+   }
+   return *this;
+}
+
+const xstring xstring::null;
+const xstring_c xstring_c::null;
